@@ -11,8 +11,8 @@ import libcurses
 from loguru import logger
 
 from tf2mon.admin import Admin
+from tf2mon.command import Command, CommandManager
 from tf2mon.conlog import Conlog
-from tf2mon.fkey import FKey, FKeyManager
 from tf2mon.gameplay import Gameplay
 from tf2mon.hacker import HackerAttr, HackerManager
 from tf2mon.msgqueue import MsgQueueManager
@@ -97,37 +97,39 @@ class Monitor:
         self.spammer = Spammer(self)
 
         # function keys
-        self.fkeys = FKeyManager()
-        self.fkeys.add(self._fkey_help("F1", curses.KEY_F1))
-        self.fkeys.add(self._fkey_motd(None, curses.KEY_F13))
-        self.fkeys.add(self._fkey_debug_flag("F2", curses.KEY_F2))
-        self.fkeys.add(self._fkey_log_level("F14", curses.KEY_F14))
-        self.fkeys.add(self._fkey_taunt_flag("F3", curses.KEY_F3))
-        self.fkeys.add(self._fkey_show_kd("F4", curses.KEY_F4))
-        self.fkeys.add(self._fkey_user_panel("F5", curses.KEY_F5))
-        self.fkeys.add(self._fkey_join_other_team("F6", curses.KEY_F6))
-        self.fkeys.add(self._fkey_sort_order("F7", curses.KEY_F7))
-        self.fkeys.add(self._fkey_log_location("F8", curses.KEY_F8))
-        self.fkeys.add(self._fkey_reset_padding("F20", curses.KEY_F20))
-        self.fkeys.add(self._fkey_grid_layout("F9", curses.KEY_F9))
-        self.fkeys.add(self._fkey_show_debug("KP_INS", curses.KEY_IC))
-        self.fkeys.add(self._fkey_single_step("KP_DEL", curses.KEY_DC))
-        self.fkeys.add(self._fkey_kick_last_cheater("[", None))
-        self.fkeys.add(self._fkey_kick_last_racist("]", None))
-        self.fkeys.add(self._fkey_kick_last_suspect("\\", None))
+        self.commands = CommandManager()
+        self.commands.bind(self._cmd_help, "F1")
+        self.commands.bind(self._cmd_motd, "Ctrl+F1")
+        self.commands.bind(self._cmd_debug_flag, "F2")
+        self.commands.bind(self._cmd_taunt_flag, "F3")
+        self.commands.bind(self._cmd_show_kd, "F4")
+        self.commands.bind(self._cmd_user_panel, "F5")
+        self.commands.bind(self._cmd_join_other_team, "F6")
+        self.commands.bind(self._cmd_sort_order, "F7")
+        self.commands.bind(self._cmd_log_location, "F8")
+        self.commands.bind(self._cmd_log_level, "Shift+F8")
+        self.commands.bind(self._cmd_reset_padding, "Ctrl+F8")
+        self.commands.bind(self._cmd_grid_layout, "F9")
+        self.commands.bind(self._cmd_show_debug, "KP_INS")
+        self.commands.bind(self._cmd_single_step, "KP_DEL")
+        self.commands.bind(self._cmd_kick_last_cheater, "[", game_only=True)
+        self.commands.bind(self._cmd_kick_last_racist, "]", game_only=True)
+        self.commands.bind(self._cmd_kick_last_suspect, "\\", game_only=True)
         # numpad
-        self.fkeys.add(self._fkey_kicks_pop("KP_HOME", curses.KEY_HOME))
-        self.fkeys.add(self._fkey_kicks_clear("KP_LEFTARROW", curses.KEY_LEFT))
-        self.fkeys.add(self._fkey_kicks_popleft("KP_END", curses.KEY_END))
-        self.fkeys.add(self._fkey_pull("KP_UPARROW", curses.KEY_UP))
-        self.fkeys.add(self._fkey_clear_queues("KP_5", curses.KEY_B2))
-        self.fkeys.add(self._fkey_push("KP_DOWNARROW", curses.KEY_DOWN))
-        self.fkeys.add(self._fkey_spams_pop("KP_PGUP", curses.KEY_PPAGE))
-        self.fkeys.add(self._fkey_spams_clear("KP_RIGHTARROW", curses.KEY_RIGHT))
-        self.fkeys.add(self._fkey_spams_popleft("KP_PGDN", curses.KEY_NPAGE))
-        #
+        self.commands.bind(self._cmd_kicks_pop, "KP_HOME")
+        self.commands.bind(self._cmd_kicks_clear, "KP_LEFTARROW")
+        self.commands.bind(self._cmd_kicks_popleft, "KP_END")
+        self.commands.bind(self._cmd_pull, "KP_UPARROW")
+        self.commands.bind(self._cmd_clear_queues, "KP_5")
+        self.commands.bind(self._cmd_push, "KP_DOWNARROW")
+        self.commands.bind(self._cmd_spams_pop, "KP_PGUP")
+        self.commands.bind(self._cmd_spams_clear, "KP_RIGHTARROW")
+        self.commands.bind(self._cmd_spams_popleft, "KP_PGDN")
+
         if self.tf2_scripts_dir.is_dir():
-            self.fkeys.create_tf2_exec_script(self.tf2_scripts_dir / "tf2-monitor-fkeys.cfg")
+            path = self.tf2_scripts_dir / "tf2-monitor-fkeys.cfg"
+            logger.info(f"Writing `{path}`")
+            path.write_text(self.commands.as_tf2_exec_script(), encoding="utf-8")
 
         # admin command handlers
         self.regex_list = self.admin.regex_list
@@ -137,7 +139,7 @@ class Monitor:
         self.regex_list += self.gameplay.regex_list
 
         # function key handlers
-        self.regex_list += self.fkeys.get_regex_list()
+        self.regex_list += self.commands.get_regex_list()
 
         #
         self.me = self.my = None
@@ -149,6 +151,7 @@ class Monitor:
         libcurses.wrapper(self._run)
 
     def _run(self, win):
+        self.commands.register_curses_handlers()
         # Build user-interface
         self.ui = UI(self, win)
         self.reset_game()
@@ -226,201 +229,165 @@ class Monitor:
     def _on_off(key, value):
         return key.upper() if value else key
 
-    def _fkey_help(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_help(self) -> Command:
 
-        return FKey(
-            cmd="HELP",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="HELP",
             status=lambda: "HELP",
             handler=lambda m: self.ui.show_help(),
         )
 
-    def _fkey_motd(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_motd(self) -> Command:
 
-        return FKey(
-            cmd="MOTD",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="MOTD",
             handler=lambda m: self.ui.show_motd(),
         )
 
-    def _fkey_debug_flag(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_debug_flag(self) -> Command:
         def _action() -> None:
             if self.toggling_enabled:
                 _ = self.ui.debug_flag.toggle
                 self.ui.show_status()
 
-        return FKey(
-            cmd="TOGGLE-DEBUG",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="TOGGLE-DEBUG",
             status=lambda: self._on_off("debug", self.ui.debug_flag.value),
             handler=lambda m: _action(),
         )
 
-    def _fkey_taunt_flag(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_taunt_flag(self) -> Command:
         def _action() -> None:
             if self.toggling_enabled:
                 _ = self.ui.taunt_flag.toggle
                 self.ui.show_status()
 
-        return FKey(
-            cmd="TOGGLE-TAUNT",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="TOGGLE-TAUNT",
             status=lambda: self._on_off("taunt", self.ui.taunt_flag.value),
             handler=lambda m: _action(),
         )
 
-    def _fkey_show_kd(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_show_kd(self) -> Command:
         def _action() -> None:
             if self.toggling_enabled:
                 _ = self.ui.show_kd.toggle
                 self.ui.show_status()
 
-        return FKey(
-            cmd="TOGGLE-KD",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="TOGGLE-KD",
             status=lambda: self._on_off("kd", self.ui.show_kd.value),
             handler=lambda m: _action(),
         )
 
-    def _fkey_user_panel(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_user_panel(self) -> Command:
         def _action() -> None:
             _ = self.ui.user_panel.toggle
             self.ui.update_display()
 
-        return FKey(
-            cmd="TOGGLE-USER-PANEL",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="TOGGLE-USER-PANEL",
             status=lambda: self.ui.user_panel.value.name,
             handler=lambda m: _action(),
         )
 
-    def _fkey_join_other_team(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_join_other_team(self) -> Command:
         def _action() -> None:
             if self.toggling_enabled:
                 self.me.assign_team(self.my.opposing_team)
                 self.ui.update_display()
 
-        return FKey(
-            cmd="SWITCH-MY-TEAM",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="SWITCH-MY-TEAM",
             status=lambda: self.my.team.name if self.my.team else "blu",
             handler=lambda m: _action(),
         )
 
-    def _fkey_sort_order(self, game_key: str, curses_key: int) -> FKey:
-        def _action() -> None:
-            self.ui.set_sort_order(self.ui.sort_order.toggle)
-            self.ui.update_display()
+    def _cmd_sort_order(self) -> Command:
 
-        return FKey(
-            cmd="TOGGLE-SORT",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="TOGGLE-SORT",
             status=lambda: self.ui.sort_order.value.name,
-            handler=lambda m: _action(),
+            handler=lambda m: (
+                self.ui.set_sort_order(self.ui.sort_order.toggle),
+                self.ui.update_display(),
+            ),
         )
 
-    def _fkey_log_level(self, game_key: str, curses_key: int) -> FKey:
-        def _action() -> None:
-            self.ui.cycle_log_level()
-            self.ui.show_status()
+    def _cmd_log_level(self) -> Command:
 
-        return FKey(
-            cmd="TOGGLE-LOG-LEVEL",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="TOGGLE-LOG-LEVEL",
             status=lambda: self.ui.log_level.value.name,
-            handler=lambda m: _action(),
+            handler=lambda m: (
+                self.ui.cycle_log_level(),
+                self.ui.show_status(),
+            ),
         )
 
-    def _fkey_log_location(self, game_key: str, curses_key: int) -> FKey:
-        def _action() -> None:
-            self.ui.cycle_log_location()
-            self.ui.show_status()
+    def _cmd_log_location(self) -> Command:
 
-        return FKey(
-            cmd="TOGGLE-LOG-LOCATION",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="TOGGLE-LOG-LOCATION",
             status=lambda: self.ui.log_location.value.name,
-            handler=lambda m: _action(),
+            handler=lambda m: (
+                self.ui.cycle_log_location(),
+                self.ui.show_status(),
+            ),
         )
 
-    def _fkey_reset_padding(self, game_key: str, curses_key: int) -> FKey:
-        def _action() -> None:
-            self.ui.logsink.reset_padding()
-            logger.info("padding reset")
+    def _cmd_reset_padding(self) -> Command:
 
-        return FKey(
-            game_key=game_key,
-            curses_key=curses_key,
-            handler=lambda m: _action(),
-            # handler=lambda m: self.ui.logsink.reset_padding(),
+        return Command(
+            name="RESET-PADDING",
+            handler=lambda m: (
+                self.ui.logsink.reset_padding(),
+                logger.info("padding reset"),
+            ),
         )
 
-    def _fkey_grid_layout(self, game_key: str, curses_key: int) -> FKey:
-        def _action() -> None:
-            self.ui.cycle_grid_layout()
+    def _cmd_grid_layout(self) -> Command:
 
-        return FKey(
-            cmd="TOGGLE-LAYOUT",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="TOGGLE-LAYOUT",
             status=lambda: self.ui.grid_layout.value.name,
-            handler=lambda m: _action(),
+            handler=lambda m: self.ui.cycle_grid_layout(),
         )
 
-    def _fkey_single_step(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_single_step(self) -> Command:
 
-        return FKey(
-            cmd="SINGLE-STEP",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="SINGLE-STEP",
             handler=lambda m: self.admin.start_single_stepping(),
         )
 
-    def _fkey_show_debug(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_show_debug(self) -> Command:
 
-        return FKey(
-            cmd="SHOW-DEBUG",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="SHOW-DEBUG",
             handler=lambda m: self.ui.show_debug(),
         )
 
-    def _fkey_kick_last_cheater(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_kick_last_cheater(self) -> Command:
 
-        return FKey(
-            cmd="KICK-LAST-CHEATER",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="KICK-LAST-CHEATER",
             status=lambda: HackerAttr.CHEATER.name,
             handler=lambda m: self.kick_my_last_killer(HackerAttr.CHEATER),
         )
 
-    def _fkey_kick_last_racist(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_kick_last_racist(self) -> Command:
 
-        return FKey(
-            cmd="KICK-LAST-RACIST",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="KICK-LAST-RACIST",
             status=lambda: HackerAttr.RACIST.name,
             handler=lambda m: self.kick_my_last_killer(HackerAttr.RACIST),
         )
 
-    def _fkey_kick_last_suspect(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_kick_last_suspect(self) -> Command:
 
-        return FKey(
-            cmd="KICK-LAST-SUSPECT",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="KICK-LAST-SUSPECT",
             status=lambda: HackerAttr.SUSPECT.name,
             handler=lambda m: self.kick_my_last_killer(HackerAttr.SUSPECT),
         )
@@ -444,12 +411,10 @@ class Monitor:
     #                   |         |         |         |
     #                   +-----------------------------+
 
-    def _fkey_kicks_pop(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_kicks_pop(self) -> Command:
 
-        return FKey(
-            cmd="KICKS-POP",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="KICKS-POP",
             handler=lambda m: (
                 self.kicks.pop(),
                 self.ui.refresh_kicks(),
@@ -457,12 +422,10 @@ class Monitor:
             action="tf2mon_kicks_pop",
         )
 
-    def _fkey_kicks_clear(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_kicks_clear(self) -> Command:
 
-        return FKey(
-            cmd="KICKS-CLEAR",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="KICKS-CLEAR",
             handler=lambda m: (
                 self.kicks.clear(),
                 self.ui.refresh_kicks(),
@@ -470,12 +433,10 @@ class Monitor:
             action="tf2mon_kicks_clear",
         )
 
-    def _fkey_kicks_popleft(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_kicks_popleft(self) -> Command:
 
-        return FKey(
-            cmd="KICKS-POPLEFT",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="KICKS-POPLEFT",
             handler=lambda m: (
                 self.kicks.popleft(),
                 self.ui.refresh_kicks(),
@@ -483,22 +444,18 @@ class Monitor:
             action="tf2mon_kicks_popleft",
         )
 
-    def _fkey_pull(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_pull(self) -> Command:
 
-        return FKey(
-            cmd="PULL",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="PULL",
             # handler=lambda m: logger.trace('pull'),
             action="tf2mon_pull",
         )
 
-    def _fkey_clear_queues(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_clear_queues(self) -> Command:
 
-        return FKey(
-            cmd="CLEAR-QUEUES",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="CLEAR-QUEUES",
             handler=lambda m: (
                 self.kicks.clear(),
                 self.ui.refresh_kicks(),
@@ -508,22 +465,18 @@ class Monitor:
             action="tf2mon_clear_queues",
         )
 
-    def _fkey_push(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_push(self) -> Command:
 
-        return FKey(
-            cmd="PUSH",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="PUSH",
             # handler=lambda m: logger.trace('push'),
             action="tf2mon_push",
         )
 
-    def _fkey_spams_pop(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_spams_pop(self) -> Command:
 
-        return FKey(
-            cmd="SPAMS-POP",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="SPAMS-POP",
             handler=lambda m: (
                 self.spams.pop(),
                 self.ui.refresh_spams(),
@@ -531,12 +484,10 @@ class Monitor:
             action="tf2mon_spams_pop",
         )
 
-    def _fkey_spams_clear(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_spams_clear(self) -> Command:
 
-        return FKey(
-            cmd="SPAMS-CLEAR",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="SPAMS-CLEAR",
             handler=lambda m: (
                 self.spams.clear(),
                 self.ui.refresh_spams(),
@@ -544,12 +495,10 @@ class Monitor:
             action="tf2mon_spams_clear",
         )
 
-    def _fkey_spams_popleft(self, game_key: str, curses_key: int) -> FKey:
+    def _cmd_spams_popleft(self) -> Command:
 
-        return FKey(
-            cmd="SPAMS-POPLEFT",
-            game_key=game_key,
-            curses_key=curses_key,
+        return Command(
+            name="SPAMS-POPLEFT",
             handler=lambda m: (
                 self.spams.popleft(),
                 self.ui.refresh_spams(),
